#include "simulation.h"
#include <iostream>
#include <chrono>
#include <thread>

Fahrstuhl::Fahrstuhl(QObject *parent) : QObject(parent), vertikalePosition(0), horizontalePosition(0), lichtErkannt1(false), gewichtsschwelleUeberschritten(false), lichtErkannt2(false), zieltemperaturErreicht(false) {}

void Fahrstuhl::bewegeZu(int zielVertikal, int zielHorizontal) {
    // Vertikale Bewegung
    while (vertikalePosition != zielVertikal) {
        if (vertikalePosition < zielVertikal) {
            vertikalePosition++;
        } else {
            vertikalePosition--;
        }
        emit fahrstuhlBewegt(vertikalePosition, horizontalePosition);
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

    // Horizontale Bewegung
    while (horizontalePosition != zielHorizontal) {
        if (horizontalePosition < zielHorizontal) {
            horizontalePosition++;
        } else {
            horizontalePosition--;
        }
        emit fahrstuhlBewegt(vertikalePosition, horizontalePosition);
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }
}

void Fahrstuhl::warteAufGewichtsschwelle() {
    std::unique_lock<std::mutex> lock(mtx);
    cv.wait(lock, [this] { return gewichtsschwelleUeberschritten.load(); });
}

void Fahrstuhl::warteAufZieltemperatur() {
    std::unique_lock<std::mutex> lock(mtx);
    cv.wait(lock, [this] { return zieltemperaturErreicht.load(); });
}

void Fahrstuhl::simuliereFahrstuhl() {
    // Bewegung zur 4. Etage
    bewegeZu(4, 0);
    // Warten auf Lichtsensor
    while (!lichtErkannt1) {
        std::this_thread::sleep_for(std::chrono::milliseconds(100));
    }

    // Warten auf Gewichtssensor
    warteAufGewichtsschwelle();

    // Bewegung zur 1. Etage
    bewegeZu(1, 0);
    // Warten auf Lichtsensor
    while (!lichtErkannt2) {
        std::this_thread::sleep_for(std::chrono::milliseconds(100));
    }

    // Bewegung nach rechts unter Block 1
    bewegeZu(1, 1);

    // Warten auf Zieltemperatur
    warteAufZieltemperatur();

    // Wasser ausgeben (Simulation)
    std::cout << "Wasser wird ausgegeben." << std::endl;

    // Bewegung zur mittleren Lücke
    bewegeZu(1, 0);

    // Bewegung zur Ausgangsposition (ganz nach unten)
    bewegeZu(0, 0);

    // Bewegung ganz nach rechts
    bewegeZu(0, 2);

    // Bewegung nach oben unter Block 6
    bewegeZu(2, 2);

    emit simulationBeendet();
}

void Fahrstuhl::starteSimulation() {
    std::thread([this] { simuliereFahrstuhl(); }).detach();
}
